/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package libpiyo;

import gabien.GaBIEn;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;

public class PiyoPiyoPlayingSample {

    // Sample indexes
    public static int[] drumTable = new int[] {0, 1, 4, -1, 2, 3, 5, -1, -1,
            -1, -1, -1};

    public static short[][] samples = new short[6][];

    public SimpleMixer.Channel channel;

    public double currentTime = 0;
    public double expectedTime = 0;

    public static void ensureAllSamplesLoaded() throws IOException {
        // Note: It may or may not be a good idea to pack this with the
        // distribution,
        // so I'm keeping it out of the source tree
        InputStream is = GaBIEn.getInFile("piyoDrums.bin");
        if (is == null)
            throw new IOException("Couldn't open piyoDrums.bin");
        DataInputStream dis = new DataInputStream(is);
        for (int drumSample = 0; drumSample < 6; drumSample++) {
            short[] sample = new short[dis.readInt() / 2];
            for (int i = 0; i < sample.length; i++) {
                int l = dis.read();
                sample[i] = (short) (dis.read() << 8);
                sample[i] |= l;
            }
            samples[drumSample] = sample;
        }
        is.close();
    }

    public static short[] getSample(int pos) {
        int drumSample = drumTable[pos / 2];
        if (drumSample == -1)
            return new short[1];
        if (samples[drumSample] == null)
            return new short[1];
        return samples[drumSample];
    }

    public PiyoPiyoPlayingSample(SimpleMixer.Channel channel, int pos, int volume, int pan, double globalVolume, double globalPitch) {
        boolean quiet = (pos % 2) == 0;
        double lValue = PiyoPiyoPlayingNote.doPan(volume, pan, true, true) * globalVolume;
        double rValue = PiyoPiyoPlayingNote.doPan(volume, pan, false, true) * globalVolume;
        if (quiet) {
            lValue *= 0.75;
            rValue *= 0.75;
        } else {
            lValue *= 0.50;
            rValue *= 0.50;
        }
        this.channel = channel;
        short[] sample = getSample(pos);
        expectedTime = sample.length / 22050.0d;
        channel.playSound(globalPitch, lValue, rValue, sample, false);
    }

    public boolean doUpdate(double deltaTime) {
        currentTime += deltaTime;
        return currentTime > (expectedTime + 0.1d);
    }
}
